#include "number.h"
#include "text.h"
#include <sstream>
#include <stdio.h>
#include "month.h"
#include "day.h"
#include "year.h"
#include "hour.h"
#include "minute.h"
#include "second.h"
#include "adverb.h"

#include <iostream>



Number::Number(Text *text) : Unit(text) {
	myclass += ":Number";
	whole_number = 0;
	decimal_number = 0;
	full_number = 0;
	starts_with_zero = 0;
	plural_next = 0;
	following_pinyin = "";
}
Number::~Number() {}

std::string Number::return_pinyin() {

  // We Manually add Pre and Post Pinyin (needed because of tone sandhi processing)
  //if (pre_chinese != NULL) { temp_pinyin  = pre_chinese->return_pinyin() + temp_pinyin; }
  //if (post_chinese != NULL) { temp_pinyin = temp_pinyin + post_chinese->return_pinyin(); }

  if (post_pinyin_spacing == 1) { return pre_pinyin + pinyin + post_pinyin + " "; }
  return pre_pinyin + pinyin + post_pinyin;

}
std::string Number::return_pinyin_no_spacing() {

  //std::string temp_pinyin = pinyin;
  //if (pre_chinese != NULL) { temp_pinyin  = pre_chinese->return_pinyin_no_spacing() + pinyin; }
  //if (post_chinese != NULL) { temp_pinyin = pinyin + post_chinese->return_pinyin_no_spacing(); }

  return pre_pinyin + pinyin + post_pinyin;

}
int Number::adjust_confidence(Text *parent, int a, int b, int c) {

  if (parent->is_category_non_recursive("Verb",a,b-1) == 1) { 
    parent->add_confidence("Verb",1,a,b-1);
  }

  return 1;

}
int Number::pre_unify(Text *parent, int a, int b, int c) {


  if (parent->return_chinese(b+1) == "" || parent->return_chinese(b+1) == "" || parent->return_chinese(b+1) == "") {
    post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
    add_pre_english("more than");
    shift_post_chinese(parent, a, b+1, 0, this);
    parent->make_only(a, b, this);
  }


  if (convert2number(chinese) == 0) { starts_with_zero = 1; }

  while (parent->is_category_non_recursive("Number", a, b+1) == 1) {
    /* We automatically generate the pinyin below, so we set the pinyin of 
     * any but the first number to "" when we add it to post_character. This 
     * ensures that when we automatically generate the pinyin (return_pinyin) 
     * we end up with something usable, and can iterate past these numbers 
     * and still get the pinyin of characters added at the end. See return_pinyin()
     * if there is any confusion.
     */
    parent->text(a, b+1, 0)->pinyin = "";
    shift_post_chinese(parent, a, b+1, "Number", this);
  }

  return 1;

}

int Number::unify(Text *parent, int a, int b, int c) {


     // Decimal Unification with NonChinese Punctuation
     if (parent->is_category_non_recursive("Number",a,b+1) == 1 &&
         parent->is_category_non_recursive("Period",a,b+1) == 1
     ) {
	shift_post_chinese(parent, a, b+1, 0, this);
	parent->make_only(a,b,this);
     }




    // Avoid Problems which Interfere with Unification
    if (parent->return_chinese(b+1) == " ") { 
      parent->make_only(a, b, this);
      parent->remove_element(a,b+1);
    }

        // Avoid Problems which Interfere with Unification
        if (parent->return_chinese(b+1) == "" || parent->return_chinese(b+1) == "" || parent->return_chinese(b+1) == "") {
          post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
          add_pre_english("more than");
          shift_post_chinese(parent, a, b+1, 0, this);
	  while (parent->is_category_non_recursive("Number",a,b+1) == 1){
    	    shift_post_chinese(parent, a, b+1, "Number", this);
	  }
          parent->make_only(a, b, this);

        }




	// Decimal Number
	if ((parent->return_chinese(b+1) == "") && parent->is_category_non_recursive("Number", a, b+2) == 1) {
		parent->text(a, b+1, 0)->pinyin = "dian3";
		shift_post_chinese(parent, a, b+1, 0, this);
		while (parent->is_category_non_recursive("Number", a, b+1) == 1) {
			shift_post_chinese(parent, a, b+1, 0, this);
		}
		parent->make_only(a, b, this);
	}
	else {
	if ((parent->return_chinese(b+1) == "" || parent->return_chinese(b+1) == ".") && parent->is_category_non_recursive("Number", a, b+2) == 1) {
		parent->text(a, b+1, 0)->pinyin = "dian3";
		shift_post_chinese(parent, a, b+1, 0, this);
		while (parent->is_category_non_recursive("Number", a, b+1) == 1) {
			shift_post_chinese(parent, a, b+1, 0, this);
		}
		parent->make_only(a, b, this);
	}
	}


	generate_numbers();
	if (decimal_number != 0) { plural_next = 1; };
	if (whole_number > 1) { plural_next = 1; };



	// Now Generate the Pinyin
	pinyin = generate_complex_pinyin(whole_number);

	if (following_pinyin == "shi2 yi4") { whole_number *= 10; following_pinyin = "yi4"; }
	if (decimal_number != 0) {
		std::string decimal_as_string = return_english();
		if (decimal_as_string.find(".") != std::string::npos) {
			decimal_as_string = decimal_as_string.substr(decimal_as_string.find(".")+1);
			pinyin += " dian3 ";
			pinyin += generate_complex_pinyin(atoi(decimal_as_string.c_str()));
		}
	}
	if (following_pinyin != "") { pinyin += following_pinyin; }



	return 1;

}

int Number::post_unify(Text *parent, int a, int b, int c) {



	//... measurements
	if (parent->is_category_non_recursive("Mment", a, b+1) == 1) {
                post_pinyin += " " + parent->text(a,b+1,"Mment")->return_pinyin();
    		add_post_english(" " + parent->text(a,b+1,"Mment")->return_english_no_spacing());
    		shift_post_chinese(parent, a, b+1, "Mment", this);   
 		parent->make_only(a, b, this);
		myclass += ":Mment";
		myclass += ":Phrase";
	}
	//... and a half
	if (parent->is_category_non_recursive("Measure", a, b+1) == 1) {
		if (parent->return_chinese(b+2) == ""){
                        post_pinyin += " " + parent->text(a,b+1,"Measure")->return_pinyin();
                        post_pinyin += " " + parent->text(a,b+2,0)->return_pinyin();
    			add_post_english("and a half");
    			shift_post_chinese(parent, a, b+1, "Measure", this);
    			shift_post_chinese(parent, a, b+2, 0, this);
    			parent->make_only(a, b, this);
			decimal_representation += 0.5;
			myclass += ":Phrase";
		}
	}

	// Shorthand for Age
	if (parent->return_chinese(b+1) == "") {
		if (parent->return_chinese(b+2) == "") {
                	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
                	post_pinyin += " " + parent->text(a,b+2,0)->return_pinyin();
			add_post_english(" years old");
			shift_post_chinese(parent, a, b+1, 0, this);
			shift_post_chinese(parent, a, b+1, 0, this);
			if (parent->return_chinese(b-1) == "") {
                		pre_pinyin += parent->text(a,b-1,0)->return_pinyin();
				if (shift_pre_chinese(parent, a, b-1, 0, this) == 1) { b--; }
				add_pre_english("is");
			}
			parent->make_only(a, b, this);
			myclass += ":Phrase";
		}
	}



	// Floor
	if (parent->return_chinese(a,b+1) == "¥") {
                post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		english = suffix(english);
		shift_post_chinese(parent, a, b+1, 0, this);
		add_post_english(" floor");
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "") {
                post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		plural_next = 0;
		add_post_english(" times");
		shift_post_chinese(parent, a, b+1, 0, this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "") {
                post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		plural_next = 0;
		add_pre_english("more than");
		add_post_english(" times");
		shift_post_chinese(parent, a, b+1, 0, this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "") {
                post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		english = suffix(english);
		add_post_english(" floor");
		shift_post_chinese(parent,a,b+1,0,this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "") {
                post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent, a,b+1,0,this);
		add_pre_english("more than ");
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(b+1) == "") {
                post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent, a,b+1,0,this);
		add_pre_english("more than ");
		add_post_english( " years");
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}



	// Weeks
	if (parent->return_chinese(a, b+1, 0) == "") {
		if (whole_number > 0 && whole_number < 20) {
                	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent, a, b+1, "Noun", this);
			add_post_english("weeks");
			myclass += ":Phrase";

			if (parent->return_chinese(a, b+1, 0) == "") {
                		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
				shift_post_chinese(parent, a, b+1, 0, this);
				add_pre_english("over the last");
				if (parent->return_chinese(a, b+1, 0) == "״") {
                			post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
					shift_post_chinese(parent, a, b+1, 0, this);
					add_pre_english("for the first time");
				}
			}
		}
	}



	// Months
	if (parent->return_chinese(b+1) == "" || parent->return_chinese(b+1) == "·") {
		if (parent->return_chinese(a, b+2) == "") {
                	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
                	post_pinyin += " " + parent->text(a,b+2,0)->return_pinyin();
			shift_post_chinese(parent, a, b+1, 0, this);
			shift_post_chinese(parent, a, b+1, 0, this);
			if (whole_number == 1) { add_post_english(" month"); }
			else { add_post_english(" months"); }
			add_pre_english("within ");
			parent->make_only(a, b, this);
			myclass += ":Phrase";
			return 1;
		}
		if (parent->return_chinese(a, b+2) == "֮") {
                	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
                	post_pinyin += " " + parent->text(a,b+2,0)->return_pinyin();
			shift_post_chinese(parent, a, b+1, 0, this);
			shift_post_chinese(parent, a, b+1, 0, this);
			if (whole_number == 1) { add_post_english(" month"); }
			else { add_post_english(" months"); }
			add_pre_english("within ");
			parent->make_only(a, b, this);
			myclass += ":Phrase";
			return 1;
		}
		if (whole_number < 13) { 
			Month *temp = new Month(this, this, parent->elements->at(0)->at(b+1)->at(0));
			parent->elements->at(a)->at(b)->push_back(temp);
			parent->make_only(a, b, temp);
			parent->remove_element(a, b+1);
			return 1;
		}
	}	
	if (parent->return_chinese(a, b+1) == "·ݻ") {
		if (whole_number < 13) { 
			Month *temp = new Month(this, this, parent->elements->at(0)->at(b+1)->at(0));
			parent->elements->at(a)->at(b)->push_back(temp);
			parent->make_only(a, b, temp);
			parent->remove_element(a, b+1);
			temp->add_post_english("meeting");
			return 1;
		}
	}


	// Days
	if (parent->return_chinese(a,b+1) == "ҹ") {
		if (whole_number < 32) { 
			Day *temp = new Day(this, this, parent->elements->at(0)->at(b+1)->at(0));
			temp->add_pre_english("the evening of the");
			parent->elements->at(a)->at(b)->push_back(temp);
			parent->make_only(a, b, temp);
			parent->remove_element(a, b+1);
			if (parent->return_chinese(a,b+1) == "") {
				shift_post_chinese(parent,a,b+1,0,temp);
			}
			return 1;
		}
	}
	if (parent->return_chinese(a, b+1) == "") {
		if (whole_number < 32) { 
			Day *temp = new Day(this, this, parent->elements->at(0)->at(b+1)->at(0));
			parent->elements->at(a)->at(b)->push_back(temp);
			parent->remove_element(a, b+1);
			if (parent->is_category_non_recursive("Number",a,b-2) == 1 && parent->return_chinese(b-1) == "-") {
				Day *temp2 = new Day(parent);
				parent->text(a,b-2,"Number")->clone_values(temp2);
				parent->elements->at(a)->at(b-2)->push_back(temp2);
				parent->make_only(a, b-2, temp2);
				temp2->post_english_spacing = 0;
				temp2->post_pinyin_spacing = 0;
			}
			parent->make_only(a, b, temp);
			return 1;
		}
	}
	if (parent->return_chinese(a, b-1) == "") {
                pre_pinyin += " " + parent->text(a,b-1,0)->return_pinyin();
		add_pre_english("the ");
		english = suffix(english);
		shift_pre_chinese(parent,a,b-1,0,this); b--;
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a, b+1) == "") {
		if (whole_number < 32 && parent->is_category_non_recursive("Number", a, b-2) == 1) { 
			Day *temp = new Day(this, this, parent->elements->at(0)->at(b+1)->at(0));
			parent->elements->at(a)->at(b)->push_back(temp);
			parent->make_only(a, b, temp);
			parent->remove_element(a, b+1);
			return 1;
		}
	}
	if (parent->return_chinese(a, b+1) == "ǰ") {
		if (whole_number < 32) { 
			Day *temp = new Day(this, this, parent->elements->at(0)->at(b+1)->at(0));
			parent->elements->at(a)->at(b)->push_back(temp);
			parent->make_only(a, b, temp);
			parent->remove_element(a, b+1);
			add_pre_english("before the ");
			return 1;
		}
	}	
	// Avoid common confusion with "Daily Newspaper"
	if (parent->return_chinese(a, b+1) == "ձ" || parent->return_chinese(a, b+1) == "ձ") {
		if (whole_number < 32) {
			parent->resegment(a, b+1, 1);
			Day *temp = new Day(this, this, parent->elements->at(0)->at(b+1)->at(0));
			parent->elements->at(a)->at(b)->push_back(temp);
			parent->make_only(a, b, temp);
			parent->remove_element(a, b+1);
			return 1;
		}
	}	








	// Percentages
	// Our first test keeps the "number" from being triggered as a number before we reach the second number
	if (parent->return_chinese(a,b+2) == "֮" && parent->return_chinese(a,b+1) == "" && parent->is_category_non_recursive("Number", a, b+3) == 1) { return 1; }
	if (parent->return_chinese(a,b-1) == "֮" && parent->return_chinese(a,b-2) == "" && parent->is_category_non_recursive("Number", a, b-3) == 1) {
		plural_next = 0;
          	pre_pinyin += parent->text(a,b-3,0)->return_pinyin() + " ";
          	pre_pinyin += parent->text(a,b-2,0)->return_pinyin() + " ";
          	pre_pinyin += parent->text(a,b-1,0)->return_pinyin() + " ";
		shift_pre_chinese(parent, a, b-1, 0, this); b--;
		shift_pre_chinese(parent, a, b-1, 0, this); b--;
		add_post_english("/");
		add_post_english(parent->text(a,b-1,"Number")->return_english());
		shift_pre_chinese(parent, a, b-1, 0, this); b--;
		parent->make_only(a,b,this);
		post_english_spacing = 0;
		myclass += ":Percent";
		myclass += ":Phrase";
		if (parent->return_chinese(a,b+1) == "") {
			parent->make_only(a,b,this);
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent,a,b+1,0,this);
			add_pre_english("around ");
		}
	}
	if (parent->return_chinese(a,b+2) == "֮" && parent->return_chinese(a,b+1) == "" && parent->return_chinese(a,b+3) == "ٷ") {
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
          	post_pinyin += " " + parent->text(a,b+2,0)->return_pinyin();
          	post_pinyin += " " + parent->text(a,b+3,0)->return_pinyin();
		shift_post_chinese(parent, a, b+1, 0, this);
		shift_post_chinese(parent, a, b+1, 0, this);
		shift_post_chinese(parent, a, b+1, 0, this);
		add_post_english(" percent");
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}





	// Years
	// Fiscal Years
	if (parent->return_chinese(a, b+1) == "") {
		parent->make_only(a,b,this);
		add_pre_english("the ");
		add_post_english(" fiscal year");
		shift_post_chinese(parent, a, b+1, 0, this);
		if (parent->return_chinese(a, b-1) == "") {
			parent->make_only(a,b-1,"Zai01");
			parent->text(a,b-1,"Zai01")->english = "in";
		}
		if (parent->return_chinese(a, b+1) == "") {
			parent->make_only(a,b+1,0);
			parent->text(a,b+1,0)->should_translate = 0;
		}
	}
	// Obscure pairings sought first
	if (parent->return_chinese(a, b-1) == "" && parent->return_chinese(a, b+1) == "") {
		if (whole_number < 300) {
			Year *temp = new Year(this, this, parent->elements->at(0)->at(b+1)->at(0));
			clone_values(temp);
			parent->elements->at(a)->at(b)->push_back(temp);
			parent->make_only(a, b, temp);
			parent->remove_element(a, b+1);
			return 1;
		}
	}
	// Shorthand References for 2001-2009
	if (starts_with_zero == 1 && whole_number < 9 && whole_number > 0) {
		if (parent->return_chinese(b+1) == "") {
			whole_number += 2000;
			english = "200" + english;
			Year *temp = new Year(this, this, parent->elements->at(0)->at(b+1)->at(0));
			parent->elements->at(a)->at(b)->push_back(temp);
			parent->make_only(a, b, temp);
			parent->remove_element(a, b+1);
			return 1;
		} else {
			Year *temp = new Year(this);
			clone_values(temp);
			parent->elements->at(a)->at(b)->push_back(temp);
			parent->make_only(a, b, temp);
			temp->integer_representation = whole_number+2000;
			if (whole_number == 0) { temp->english = "2000"; }
			if (whole_number == 1) { temp->english = "2001"; }
			if (whole_number == 2) { temp->english = "2002"; }
			if (whole_number == 3) { temp->english = "2003"; }
			if (whole_number == 4) { temp->english = "2004"; }
			if (whole_number == 5) { temp->english = "2005"; }
			if (whole_number == 6) { temp->english = "2006"; }
			if (whole_number == 7) { temp->english = "2007"; }
			if (whole_number == 8) { temp->english = "2008"; }
			if (whole_number == 9) { temp->english = "2009"; }
			return 1;
		}
	}


	// Reference to Proximate Years
	if (parent->return_chinese(a, b+1) == "" || parent->return_chinese(a, b+1) == "") {
		if (parent->return_chinese(a,b+3) == "" && parent->return_chinese(a,b+2) == "") {
          	        post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
          	        post_pinyin += " " + parent->text(a,b+2,0)->return_pinyin();
          	        post_pinyin += " " + parent->text(a,b+3,0)->return_pinyin();
			shift_post_chinese(parent, a, b+1, 0, this);
			shift_post_chinese(parent, a, b+1, 0, this);
			shift_post_chinese(parent, a, b+1, 0, this);
			add_pre_english("over more than the last ");
			if (whole_number == 1) { add_post_english(" year"); } 
			else { add_post_english(" years"); }
			parent->make_only(a,b, this);
			confidence += 10;
			plural_next = 0;
			parent->make_only(a,b,this);
			myclass += ":Phrase";
			return 1;
		}
		if (parent->return_chinese(a,b+2) == ""){
			if (parent->return_chinese(a, b+3) == "״") {
          	        	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
          	        	post_pinyin += " " + parent->text(a,b+2,0)->return_pinyin();
          	        	post_pinyin += " " + parent->text(a,b+3,0)->return_pinyin();
				shift_post_chinese(parent, a, b+1, 0, this);
				shift_post_chinese(parent, a, b+1, 0, this);
				shift_post_chinese(parent, a, b+1, 0, this);
				add_pre_english("for the first time in ");
				if (whole_number == 1) { add_post_english(" year"); }
				else { add_post_english(" years"); }
				myclass += ":Time";
				parent->make_only(a,b, this);
				myclass += ":Phrase";
				plural_next = 0;
				return 1;
			}
          	        post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
          	        post_pinyin += " " + parent->text(a,b+2,0)->return_pinyin();
			shift_post_chinese(parent, a, b+1, 0, this);
			shift_post_chinese(parent, a, b+1, 0, this);
			add_pre_english("in the last ");
			if (whole_number == 1) { add_post_english(" year"); }
			else { add_post_english(" years"); }
			confidence += 10;
			plural_next = 0;
			parent->make_only(a,b,this);
			myclass += ":Phrase";
			return 1;
		}
		if (parent->return_chinese(a,b+2) == "֮") {
          	        post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
          	        post_pinyin += " " + parent->text(a,b+2,0)->return_pinyin();
			shift_post_chinese(parent, a, b+1, 0, this);
			shift_post_chinese(parent, a, b+1, 0, this);
			add_pre_english("within ");
			if (whole_number == 1) { add_post_english(" year"); }
			else { add_post_english(" years"); }
			plural_next = 0;
			myclass += ":Time";
			parent->make_only(a,b, this);
			myclass += ":Phrase";
			return 1;
		}
		if (parent->return_chinese(a, b+2) == "") {
          	        post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
          	        post_pinyin += " " + parent->text(a,b+2,0)->return_pinyin();
			shift_post_chinese(parent, a, b+1, 0, this);
			shift_post_chinese(parent, a, b+1, 0, this);
			add_pre_english("within ");
			if (whole_number == 1) { add_post_english(" year"); }
			else { add_post_english(" years"); }
			plural_next = 0;
			myclass += ":Time";
			myclass += ":Phrase";
			parent->make_only(a,b, this);
			plural_next = 0;
			return 1;
		}
		if (whole_number > 1800 && whole_number < 2200) {
			Year *temp = new Year(this, this, parent->elements->at(0)->at(b+1)->at(0));
			parent->elements->at(a)->at(b)->push_back(temp);
			parent->remove_element(a, b+1);
			temp->confidence+=10;
			parent->make_only(a,b,temp);
			return 1;
		}
		if (parent->return_chinese(a,b-1) == "ڹȥ") {
          	        post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
          	        pre_pinyin += " " + parent->text(a,b-1,0)->return_pinyin();
			shift_pre_chinese(parent,a,b-1,0,this); b--;
			shift_post_chinese(parent,a,b+1,0,this);
			add_pre_english("in the past ");
			if (whole_number == 1) { add_post_english(" year"); }
			else { add_post_english(" years"); }
			parent->make_only(a,b,this);
			myclass += ":Phrase";
			myclass += ":Time:Range";
			return 1;
		}
		if (parent->return_chinese(a,b-1) == "δ") {
          	        post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
          	        pre_pinyin += " " + parent->text(a,b-1,0)->return_pinyin();
			shift_pre_chinese(parent,a,b-1,0,this); b--;
			shift_post_chinese(parent,a,b+1,0,this);
			add_pre_english("in the future ");
			if (whole_number == 1) { add_post_english(" year"); }
			else { add_post_english(" years"); }
			parent->make_only(a,b,this);
			myclass += ":Phrase";
			myclass += ":Time:Range";
			return 1;
		}
	}
	if (whole_number > 1900 && whole_number < 2080) {
		Year *temp = new Year(this);
		clone_values(temp);
		parent->elements->at(a)->at(b)->push_back(temp);
		temp->confidence+=5;
		if (parent->return_chinese(a,b+1) == "ǰ") {
			shift_post_chinese(parent,a,b+1,0,temp);
			temp->add_pre_english("before the start of ");
			parent->make_only(a,b,temp);
			return 1;
		}
	}



	if (parent->return_chinese(b-1) == "Ϊ" && parent->is_category("Noun",a,b+1) == 1) {
		plural_next = 0;
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
          	pre_pinyin += " " + parent->text(a,b-1,0)->return_pinyin();
		add_post_english(parent->text(a,b+1,"Noun")->return_english_no_spacing() + "-long");
		shift_pre_chinese(parent,a,b-1,0,this); b--;
		shift_post_chinese(parent,a,b+1,0,this);
		myclass += ":Adjective";
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}


	// Time with No Reference to Hours
	if (parent->is_category_non_recursive("Colon",a,b+1) == 1) {
		if (parent->is_category_non_recursive("Number",a,b+2) == 1) {
			if (whole_number < 25) {
          			post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
				shift_post_chinese(parent, a, b+1, 0, this);
				add_post_english(":");
				add_post_english(parent->text(a,b+1,"Number")->return_english());
				shift_post_chinese(parent, a, b+1, 0, this);

				if (parent->is_category_non_recursive("Colon",a,b+1) == 1) {
					if (parent->is_category_non_recursive("Colon",a,b+1) == 1) {
          					post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
						shift_post_chinese(parent, a, b+1, 0, this);
						add_post_english(":");
						add_post_english(parent->text(a,b+1,"Number")->return_english());
          					post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
						shift_post_chinese(parent, a, b+1, 0, this);
					}
				}

				if (parent->return_chinese(a,b-1,0) == "") {
          				pre_pinyin += " " + parent->text(a,b-1,0)->return_pinyin();
					shift_pre_chinese(parent, a, b-1, 0, this); b--;
					add_post_english(" in the afternoon");
				}
				if (parent->return_chinese(a,b-1,0) == "") {
          				pre_pinyin += " " + parent->text(a,b-1,0)->return_pinyin();
					shift_pre_chinese(parent, a, b-1, 0, this); b--;
					add_post_english(" in the afternoon");
				}
				if (parent->return_chinese(a,b-1,0) == "") {
          				pre_pinyin += " " + parent->text(a,b-1,0)->return_pinyin();
					shift_pre_chinese(parent, a, b-1, 0, this); b--;
					add_post_english(" in the afternoon");
				} 
				if (parent->return_chinese(a,b-1,0) == "ҹ") {
          				pre_pinyin += " " + parent->text(a,b-1,0)->return_pinyin();
					shift_pre_chinese(parent, a, b-1, 0, this); b--;
					add_post_english(" in the afternoon");
				}

				parent->make_only(a,b,this);
				myclass += ":Phrase";
				myclass += ":Time";

			}
		}
	}



	// Hours
	if (parent->return_chinese(a,b+1) == "ʱ") {
		Hour *temp = new Hour(this, this, parent->elements->at(0)->at(b+1)->at(0));
		parent->elements->at(a)->at(b)->push_back(temp);
		parent->make_only(a, b, temp);
		parent->remove_element(a, b+1);
		parent->add_post_english("or so");
		return 1;
	}
	if (whole_number > 0 && whole_number < 24 && (parent->return_chinese(a,b+1) == "ʱ" || parent->return_chinese(a,b+1) == "" || parent->return_chinese(a,b+1) == "")) {
		Hour *temp = new Hour(this, this, parent->elements->at(0)->at(b+1)->at(0));
		parent->elements->at(a)->at(b)->push_back(temp);
		parent->make_only(a, b, temp);
		parent->remove_element(a, b+1);
		return 1;
	}
	if (whole_number < 0 && whole_number > 24 && (parent->return_chinese(a,b+1) == "")) {
		Hour *temp = new Hour(this, this, parent->elements->at(0)->at(b+1)->at(0));
		parent->elements->at(a)->at(b)->push_back(temp);
		parent->make_only(a, b, temp);
		parent->remove_element(a, b+1);
		temp->add_post_english("on the dot");
		return 1;
	}
	if (parent->return_chinese(a,b+1) == "Сʱ") {
		if (whole_number > 1) {	add_post_english(" hours"); }
		if (whole_number == 1){	add_post_english(" hour");  }
		parent->make_only(a, b, this);
		myclass += ":Phrase";
		shift_post_chinese(parent, a, b+1, 0, this);
		if (parent->return_chinese(a,b+1) == "") {
			shift_post_chinese(parent,a,b+1,0,this);
			parent->make_only(a,b,this);
			add_pre_english("after");
		}
	}





	// Minutes
	if (parent->return_chinese(a,b+1) == "") {
		if (parent->return_chinese(a,b+2) != "֮") {
			Text *temp1 = new Text(this, parent->elements->at(0)->at(b+1)->at(0), parent->elements->at(0)->at(b+2)->at(0));
			Text *temp2 = new Minute(this, this, temp1);
			delete temp1;
			parent->elements->at(a)->at(b)->push_back(temp2);
			parent->make_only(a, b, temp2);
			parent->remove_element(a, b+1);
			parent->remove_element(a, b+1);
			return 1;
		}	
	}
	// Seconds, Minutes and Hours
	if (parent->return_chinese(a,b+1) == "") {
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		if (whole_number == 1) { add_post_english(" minute"); }
		if (whole_number > 1) { add_post_english(" minutes"); }
		myclass += ":Minutes";
		myclass += ":Phrase";
		parent->make_only(a,b,this);
	}









	// Decades
	if (whole_number == 10 ||
	    whole_number == 20 ||
	    whole_number == 30 ||
	    whole_number == 40 ||
	    whole_number == 50 ||
	    whole_number == 60 ||
	    whole_number == 70 ||
	    whole_number == 80 ||
	    whole_number == 90) {
		if (parent->return_chinese(a,b+1) == "") {
			plural_next = 0;
			english += "s";
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent, a, b+1, 0, this);
			if (parent->return_chinese(a,b+1) == "") {
				add_pre_english("the middle of the");
				shift_post_chinese(parent, a, b+1, 0, this);
			}
			if (parent->return_chinese(a,b+1) == "") {
          			post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
				add_pre_english("the beginning of the");
				shift_post_chinese(parent, a, b+1, 0, this);
			}
			if (parent->return_chinese(a,b+1) == "ĩ") {
				add_pre_english("the end of the");
          			post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
				shift_post_chinese(parent, a, b+1, 0, this);
			}
			if (parent->return_chinese(a,b-1) == "") {
				add_post_english("of this century");
          			pre_pinyin += " " + parent->text(a,b-1,0)->return_pinyin();
				shift_pre_chinese(parent, a, b-1, 0, this);
				b--;
				if (parent->return_chinese(a,b-1) == "") {
					add_post_english("of this century");
          				pre_pinyin += " " + parent->text(a,b-1,0)->return_pinyin();
					shift_pre_chinese(parent, a, b-1, 0, this);
					b--;
				}
			}
			if (parent->return_chinese(a,b-1) == "") {
				add_post_english("of the last century");
          			pre_pinyin += " " + parent->text(a,b-1,0)->return_pinyin();
				shift_pre_chinese(parent, a, b-1, 0, this);
				b--;
			}
			if (parent->return_chinese(a,b-1) == "") {
				add_pre_english("entering");
          			pre_pinyin += " " + parent->text(a,b-1,0)->return_pinyin();
				shift_pre_chinese(parent, a, b-1, 0, this);
				b--;
			}
			if (pre_english == NULL) {
				add_pre_english("in the ");
				if (
					parent->is_category_non_recursive("Comma",a,b-1) == 1 &&
					parent->is_category_non_recursive("Number",a,b-2) == 1
				) {
					int tempx = parent->text(a,b-2,"Number")->integer_representation;
					if (tempx < 10) {
						add_post_english(" and the decade before that");
          					pre_pinyin += parent->text(a,b-2,0)->return_pinyin() + " ";
          					pre_pinyin += parent->text(a,b-1,0)->return_pinyin() + " ";
						shift_pre_chinese(parent,a,b-1,0,this); b--;
						shift_pre_chinese(parent,a,b-1,0,this); b--;
						parent->make_only(a,b,this);
					}
				}
			}
			parent->make_only(a,b,this);
			myclass += ":Phrase";
		}
	}











	// Number of Months
	if (whole_number < 13 && parent->is_category_non_recursive("Measure", a, b+1) == 1 && parent->return_chinese(a,b+2) == "") {
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
          	post_pinyin += " " + parent->text(a,b+2,0)->return_pinyin();
		shift_post_chinese(parent, a, b+1, 0, this);
		shift_post_chinese(parent, a, b+1, 0, this);
		if (whole_number == 1) { add_post_english("month"); }
		else { add_post_english("months"); }

		if (parent->return_chinese(a,b+1) == "") {
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent, a, b+1, 0, this);
			add_pre_english("within");
		}
			
		if (parent->return_chinese(a,b-1) == "ǰ") { 
          		pre_pinyin += " " + parent->text(a,b-1,0)->return_pinyin();
			shift_pre_chinese(parent, a, b-1, 0, this);
			b--;
			add_pre_english("the previous");
			if (parent->return_chinese(a, b-1) == "") {
          			pre_pinyin += " " + parent->text(a,b-1,0)->return_pinyin();
				shift_pre_chinese(parent, a, b-1, 0, this);
				b--;
				add_post_english("of this year");
			}
		}
		if (parent->return_chinese(a,b+1) == "ǰ") { 
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent, a, b+1, 0, this);
			add_post_english("before");
		}
		if (parent->return_chinese(a,b+1) == "Ժ") {
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent, a, b+1, 0, this);
			add_post_english("after");
		}
		if (parent->return_chinese(a,b+1) == "֮ǰ") {
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent, a, b+1, 0, this);
			add_post_english("before");
		}
		if (parent->return_chinese(a,b+1) == "Ժ") {
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent, a, b+1, 0, this);
			add_post_english("after");
		}
		if (parent->return_chinese(a,b-1) == "ͷ") {
          		pre_pinyin += " " + parent->text(a,b-1,0)->return_pinyin();
			shift_pre_chinese(parent, a, b-1, 0, this);
			b--;
			add_pre_english("first");
		}
		myclass += ":Phrase";
		parent->make_only(a,b,this);
	}











	if (parent->return_chinese(a,b-1) == "") {
          	pre_pinyin += " " + parent->text(a,b-1,0)->return_pinyin();
		shift_pre_chinese(parent, a, b-1, 0, this); b--;
		add_pre_english("and");
		add_post_english("other");
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b-1) == "ȥ") {
          	pre_pinyin += " " + parent->text(a,b-1,0)->return_pinyin();
		shift_pre_chinese(parent, a, b-1, 0, this); b--;
		add_pre_english("the past");
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}




	// Repeated Numbers
	if (decimal_number == 0) {
		int rem = 0;
		int suspect = 0; 
		rem = (whole_number-89)%100; if (rem == 0) {  suspect = 1;  }
		rem = (whole_number-78)%100; if (rem == 0) {  suspect = 1;  }
		rem = (whole_number-67)%100; if (rem == 0) {  suspect = 1;  }
		rem = (whole_number-56)%100; if (rem == 0) {  suspect = 1;  }
		rem = (whole_number-45)%100; if (rem == 0) {  suspect = 1;  }
		rem = (whole_number-34)%100; if (rem == 0) {  suspect = 1;  }
		rem = (whole_number-23)%100; if (rem == 0) {  suspect = 1;  }
		rem = (whole_number-12)%100; if (rem == 0) {  suspect = 1;  }

		if (suspect == 1) {
			// Opens Can of Worms, But We Have Found Our Sequential Doubles
			// One and Two, Two or Three, Three or Four, etc.
		}
	}










	// Days and Months by reference to Numbers
	// GB2312 is used as return_chinese(), so a length of four implies a character value of 2 chars
	if ((whole_number < 100 && whole_number > 10) && return_chinese().length() == 4) {

		// GB2312 Specific
		std::string nword = return_chinese().substr(0, 2);
		std::string oword = return_chinese().substr(2);
		std::string possiblem = "";
		std::string possibled = "";

		if (parent->return_chinese(a,b+1) == "" || (parent->is_category_non_recursive("Punctuation",a,b-1) == 1 && parent->is_category_non_recursive("Punctuation",a,b+1) == 1)) {
			if (parent->return_chinese(a,b-1) == "") {

				if (nword == "һ") { possiblem = "January"; }
				if (nword == "") { possiblem = "Februrary"; }
				if (nword == "") { possiblem = "March"; }
				if (nword == "") { possiblem = "April"; }
				if (nword == "") { possiblem = "May"; }
				if (nword == "") { possiblem = "June"; }
				if (nword == "") { possiblem = "July"; }
				if (nword == "") { possiblem = "August"; }
				if (nword == "") { possiblem = "September"; }
				if (nword == "ʮ") { possiblem = "October"; }

				if (oword == "һ") { possibled = "1st"; }
				if (oword == "") { possibled = "2nd"; }
				if (oword == "") { possibled = "3rd"; }
				if (oword == "") { possibled = "4th"; }
				if (oword == "") { possibled = "5th"; }
				if (oword == "") { possibled = "6th"; }
				if (oword == "") { possibled = "7th"; }
				if (oword == "") { possibled = "8th"; }
				if (oword == "") { possibled = "9th"; }
				if (oword == "ʮ") { possibled = "10th"; }
		
				if (possibled != "" && possiblem != "" && parent->is_category_non_recursive("Measure",a,b+1) != 1) {
					Date *temp = new Date(this);
					clone_values(temp);
					temp->english = possiblem + " " + possibled;
					temp->myclass += ":Month:Day";
					parent->elements->at(a)->at(b)->push_back(temp);
					parent->make_only(a,b,temp);
					return 1;
				}
			}
		}
	}
	if (parent->is_category_non_recursive("Dot",a,b+1) == 1 && parent->is_category_non_recursive("Number",a,b+2) == 1) {
		english = return_english_no_spacing() + "-" + parent->text(a,b+2,"Number")->return_english_no_spacing();
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
          	post_pinyin += " " + parent->text(a,b+2,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		shift_post_chinese(parent,a,b+1,0,this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
		plural_next = 0;
	}



	if (parent->return_chinese(a,b+1) == "") {
		plural_next = 0;
		english = suffix(english);
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		parent->make_only(a,b, this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "") {
		plural_next = 0;
		english = suffix(english);
		add_post_english("round ");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		myclass += ":Phrase";
		parent->make_only(a,b, this);
	}
	if (parent->return_chinese(a,b+1) == "") {
		add_pre_english("more or less ");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent, a, b+1, 0, this);
		parent->make_only(a,b, this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "") {
		add_post_english("class");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent, a, b+1, 0, this);
		parent->make_only(a,b, this);
		plural_next = 0;
		myclass += ":Phrase";
	}

	
	if (parent->return_chinese(a,b+1) == "") {
		english = suffix(english);
		add_post_english(" anniversary");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		if (parent->return_chinese(a,b-1) == "") {
			parent->make_only(a,b-1,"Noun");
			parent->text(a,b-1,0)->english = "establishment";
			parent->make_only(a,b-1,0);
		}
		parent->make_only(a,b,this);
		myclass += ":Phrase";
		plural_next = 0;
	}





	// Percentages
	if (parent->return_chinese(a,b+1) == "" || parent->return_chinese(a,b+1) == "%") { 
		add_post_english(" percent");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent, a, b+1, 0, this);
		plural_next = 0;
		if (parent->return_chinese(a,b+1) == "") {
			parent->make_only(a,b,this);
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent,a,b+1,0,this);
			add_pre_english("around ");
		}
		if (parent->is_category_non_recursive("Noun", a, b+1) == 1) {
			add_post_english(" of");
		}
		parent->make_only(a,b,this);
		myclass += ":Phrase";
		myclass += ":Percent";

	}











	// Do Not Plural in Some Cases
	if (parent->return_chinese(a,b+1,0) == "") { plural_next = 0; }



	// Measurement Words Following
	if (parent->is_category_non_recursive("Measurement",a,b+1) == 1) {
		plural_next = 0;
		parent->make_only(a,b+1,"Measurement");
	}


	// Centuries and Anniversaries
	if (parent->return_chinese(a,b+1) == "" || 
	    parent->return_chinese(a,b+1) == "" ||
	    parent->return_chinese(a,b+1) == "" || 
	    parent->return_chinese(a,b+1) == "") {
		plural_next = 0;
		english = suffix(english);
		confidence += 10;
	}











	// / Precedes 
	if (parent->return_chinese(a,b-1) == "" || parent->return_chinese(a,b-1) == "") {

          	pre_pinyin += parent->text(a,b-1,0)->return_pinyin() + " ";
		shift_pre_chinese(parent, a, b-1, 0, this); b--;
		parent->make_only(a,b,this);		
		myclass += ":Phrase";
		english = suffix(return_english_no_spacing());
		plural_next = 0;

		// /
		if (parent->return_chinese(a,b+1) == "") {
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent, a, b+1, 0, this);
			if (parent->is_category_non_recursive("Verb",a,b+1) == 1) {
				Adverb *temp = new Adverb(this);
				clone_values(temp);
				temp->english = "for the " + return_english() + " time";
				temp->myclass += ":PreD";
				parent->elements->at(a)->at(b)->push_back(temp);
				parent->make_only(a,b,temp);
				return 1;
			} 
			else {
				Text *temp = new Text(this);
				clone_values(temp);
				temp->english = return_english();
				temp->myclass += "Unit:PreNoun";
				parent->elements->at(a)->at(b)->push_back(temp);
				parent->make_only(a,b,temp);
				return 1;
			}
		}
		if (parent->return_chinese(a,b+1) == "") {
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent,a,b+1,0,this);
		}
		if (parent->return_chinese(a,b+1) == "") {
			english = suffix(english);
		}
		if (parent->return_chinese(a,b+1) == "") {
			add_post_english("generation");
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent,a,b+1,0,this);
		}
		if (parent->is_category_non_recursive("Measure",a,b+1) == 1) {
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent,a,b+1,0,this);
			myclass += ":Measure:PreNoun";
			parent->make_only(a,b,this);
		}
	}






	// Currencies
	if (parent->return_chinese(a,b-1) == "") {
          	pre_pinyin += " " + parent->text(a,b-1,0)->return_pinyin();
		shift_pre_chinese(parent,a,b-1,0,this); b--;
		add_post_english(" RMB");
		parent->make_only(a,b,this);
		myclass += ":Phrase";
		if (parent->return_chinese(a,b+1) == "Ԫ") {
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent, a, b+1, 0, this);
		}
		plural_next = 0;
	}
	if (parent->return_chinese(a,b-1) == "Ԫ") {
          	pre_pinyin += " " + parent->text(a,b-1,0)->return_pinyin();
		shift_pre_chinese(parent,a,b-1,0,this); b--;
		add_post_english(" USD");
		parent->make_only(a,b,this);
		myclass += ":Phrase";
		plural_next = 0;
	}
	if (parent->return_chinese(a,b-1) == "Ԫ") {
          	pre_pinyin += " " + parent->text(a,b-1,0)->return_pinyin();
		shift_pre_chinese(parent,a,b-1,0,this); b--;
		add_post_english(" CND");
		parent->make_only(a,b,this);
		myclass += ":Phrase";
		plural_next = 0;
	}
	if (parent->return_chinese(a,b-1) == "Ԫ") {
          	pre_pinyin += " " + parent->text(a,b-1,0)->return_pinyin();
		shift_pre_chinese(parent,a,b-1,0,this); b--;
		add_post_english(" YEN");
		parent->make_only(a,b,this);
		myclass += ":Phrase";
		plural_next = 0;
	}
	if (parent->return_chinese(a,b-1) == "ŷԪ") {
          	pre_pinyin += " " + parent->text(a,b-1,0)->return_pinyin();
		shift_pre_chinese(parent,a,b-1,0,this); b--;
		add_post_english(" EURO");
		parent->make_only(a,b,this);
		myclass += ":Phrase";
		plural_next = 0;
	}



	// Measure Words & Misc
	if (parent->return_chinese(a,b+1) == "Ԫ") {
		plural_next = 0;
		add_post_english(" yuan");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent, a, b+1, 0, this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
		if (parent->return_chinese(a, b+1) == "") { shift_post_chinese(parent, a, b+1, 0, this); }
		if (parent->return_chinese(a,b+1) == "Ǯ") { shift_post_chinese(parent, a, b+1, 0, this); }
	}
	if (parent->return_chinese(a,b+1) == "Ԫ") {
		plural_next = 0;
		add_post_english(" dollars");
		add_pre_english("more than");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent, a, b+1, 0, this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "") {
		plural_next = 0;
		add_post_english(" kuai");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent, a, b+1, 0, this);
		if (parent->return_chinese(a, b+1) == "") { shift_post_chinese(parent, a, b+1, 0, this); }
		if (parent->return_chinese(a,b+1) == "Ǯ") { shift_post_chinese(parent, a, b+1, 0, this); }
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "ë") {
		plural_next = 0;
		add_post_english(" mao");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent, a, b+1, 0, this);
		if (parent->return_chinese(a, b+1) == "") { shift_post_chinese(parent, a, b+1, 0, this); }
		if (parent->return_chinese(a,b+1) == "Ǯ") { shift_post_chinese(parent, a, b+1, 0, this); }
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "" && parent->return_chinese(a,b+2) == "֮") {
		plural_next = 0;
		add_post_english(" fen");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
          	post_pinyin += " " + parent->text(a,b+2,0)->return_pinyin();
		shift_post_chinese(parent, a, b+1, 0, this);
		shift_post_chinese(parent, a, b+1, 0, this);
		if (parent->return_chinese(a, b+1) == "" || parent->return_chinese(a,b+1) == "Ǯ") { 
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent, a, b+1, 0, this); 
		}
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "") {
		plural_next = 0;
		add_post_english(" dollars");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent, a, b+1, 0, this);
		add_pre_english(" ");
		add_pre_english("more than");
		if (parent->return_chinese(a,b+1) == "Ǯ") { shift_post_chinese(parent, a, b+1, 0, this); }
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "¥") {
		plural_next = 0;
		add_post_english(" story building");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent, a, b+1, 0, this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "¥") {
		plural_next = 0;
		add_post_english(" story building");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent, a, b+1, 0, this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}








	if (parent->return_chinese(a,b+1) == "") {
		if (parent->is_category_non_recursive("Number",a,b+2) != 1) {
			if (whole_number == 1) { english = "ten percent"; }
			if (whole_number == 2) { english = "twenty percent"; }
			if (whole_number == 3) { english = "thirty percent"; }
			if (whole_number == 4) { english = "forty percent"; }
			if (whole_number == 5) { english = "fifty percent"; }
			if (whole_number == 6) { english = "sixty percent"; }
			if (whole_number == 7) { english = "seventy percent"; }
			if (whole_number == 8) { english = "eighty percent"; }
			if (whole_number == 9) { english = "ninety percent"; }
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent,a,b+1,0,this);
			parent->make_only(a,b,this);
			myclass += ":Phrase";
		}
	}
	if (parent->return_chinese(a,b+1) == "") {
		add_post_english(" or more");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "") {
		add_pre_english("upwards of ");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "") {
		add_pre_english("downwards of ");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "" && parent->is_category_non_recursive("Number",a,b+2) == 1) {
		add_post_english(" to");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		add_post_english(" ");
		add_post_english(parent->text(a,b+2,"Number")->return_english());
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		parent->make_only(a,b,this);
		myclass += ":Range";
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "" && parent->is_category_non_recursive("Date",a,b) != 1) {
		if (parent->is_category_non_recursive("NonChinese",a,b-1) != 1) {
			add_post_english("downwards of");
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent,a,b+1,0,this);
			parent->make_only(a,b,this);
			myclass += ":Phrase";
			return 1;
		}
	}
	if (parent->return_chinese(a,b+1) == "") {
		add_post_english(" over");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "") {
		add_pre_english("more than ");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "") {
		add_post_english(" more than");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "") {
		add_post_english(" more than");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "") {
		add_post_english(" less than");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "ʱ") {
		add_pre_english("over the last ");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "") {
		add_pre_english("number ");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
		plural_next = 0;
	}
	if (parent->return_chinese(a,b+1) == "") {
		add_post_english(" room");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		parent->make_only(a,b,this);
		plural_next = 0;
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b-1) == "") {
		add_pre_english("over the last ");
          	pre_pinyin += parent->text(a,b-1,0)->return_pinyin() + " ";
		shift_pre_chinese(parent,a,b-1,0,this); b--;
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "") {
		add_pre_english("over the past ");
		if (whole_number == 1) { add_post_english(" year"); } else { add_post_english(" years"); }
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "" || parent->return_chinese(a,b+1) == "" || parent->return_chinese(a,b+1) == "") {
		add_post_english(" years old");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		post_chinese->pre_pinyin_spacing = 1;
		post_chinese->post_pinyin_spacing = 1;
		parent->make_only(a,b,this);
		myclass += ":Phrase";
		myclass += ":Age";
		if (parent->return_chinese(b-1) == "") {
          		pre_pinyin += parent->text(a,b-1,0)->return_pinyin();
			if (shift_pre_chinese(parent, a, b-1, 0, this) == 1) { b--; }
			add_pre_english("at the age of");
		}
		if (parent->return_chinese(a,b+1) == "") {
			add_pre_english("upwards of ");
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent,a,b+1,0,this);
			parent->make_only(a,b,this);
		}
		if (parent->return_chinese(a,b+1) == "") {
			add_pre_english("less than ");
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent,a,b+1,0,this);
			parent->make_only(a,b,this);
		}
		if (parent->return_chinese(a,b-1) == "") {
			add_post_english("this year ");
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent,a,b+1,0,this);
			parent->make_only(a,b,this);
			myclass += ":Phrase";
		}
	}






	// Measure Words & Misc
	if (parent->return_chinese(a,b+1) == "Ԫ") {
		plural_next = 0;
		add_post_english(" yuan");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "϶") {
		plural_next = 0;
		if (whole_number != 1) { add_post_english(" degrees Celcius"); }
		else { add_post_english(" degree Celcius"); }
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "") {
		plural_next = 0;
		add_post_english(" kuai");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "ë") {
		plural_next = 0;
		add_post_english(" mao");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "" == 1 && parent->return_chinese(a,b+2) != "֮") {
		plural_next = 0;
		add_post_english(" fen");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b+1) == "") {
		plural_next = 0;

		if (whole_number == 1 || whole_number == 2 || whole_number == 3 || whole_number == 4 || whole_number == 5 || whole_number == 6 || whole_number == 7 || whole_number == 8 || whole_number == 9) {
			add_post_english(" percent");
			add_pre_english("discounted to ");
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent,a,b+1,0,this);
			parent->make_only(a,b,this);
			myclass += ":Phrase";
			english += return_english() + "0";
		}
	}
	if (parent->return_chinese(a,b-1) == "" && parent->is_category_non_recursive("Number",a,b-2) == 1) {
		if (parent->is_category_non_recursive("Time",a,b) == -1 && parent->is_category_non_recursive("Month",a,b) == -1) {
			add_pre_english("to ");
          		pre_pinyin += parent->text(a,b-1,0)->return_pinyin() + " ";
			shift_pre_chinese(parent,a,b-1,0,this); b--;
			parent->make_only(a,b,this);
			myclass += ":Phrase";
		}
	}

	if (parent->return_chinese(a,b+1) == "") {
		add_pre_english("around ");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}











	if (parent->return_chinese(a,b+1) == "") {
		plural_next = 0;
		add_pre_english("level ");
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		if  (parent->return_chinese(a,b-1) == "") {
          		pre_pinyin += parent->text(a,b-1,0)->return_pinyin() + " ";
			shift_pre_chinese(parent,a,b-1,0,this); b--;
			add_post_english(" on the Richter scale");
			if (parent->return_chinese(a,b+1) == "ǿ") {
				add_pre_english("an intense earthquake measuring ");
          			post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
				shift_post_chinese(parent,a,b+1,0,this);
			}
			if (parent->return_chinese(a,b+1) == "") {
				add_pre_english("an earthquake measuring ");
          			post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
				shift_post_chinese(parent,a,b+1,0,this);
			}
		}
		if (parent->return_chinese(a,b+1) == "") {
			add_pre_english("more than ");
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent,a,b+1,0,this);
		}
		if (parent->return_chinese(a,b+1) == "") {
			add_pre_english("less than ");
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent,a,b+1,0,this);
		}
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b-1) == "" && parent->return_chinese(a,b+1) == "") {
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
          	pre_pinyin += parent->text(a,b-1,0)->return_pinyin() + " ";
		shift_post_chinese(parent,a,b+1,0,this);
		shift_pre_chinese(parent,a,b-1,0,this); b--;
		if (whole_number > 1) { add_post_english(" years"); }
		else { add_post_english(" year"); }
		add_post_english(" punishment");
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b-1) == "ͽ" && parent->return_chinese(a,b+1) == "") {
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
          	pre_pinyin += parent->text(a,b-1,0)->return_pinyin() + " ";
		shift_post_chinese(parent,a,b+1,0,this);
		shift_pre_chinese(parent,a,b-1,0,this); b--;
		if (whole_number > 1) { add_post_english(" years"); }
		else { add_post_english(" year"); }
		add_post_english(" limited imprisonment");
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}
	if (parent->is_category_non_recursive("Measure",a,b+1) == 1 && parent->return_chinese(a,b+2) == "") {
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		add_pre_english("more than ");
		if (parent->return_chinese(a,b+1) == "") {
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent,a,b+1,0,this);
			add_pre_english("in the last ");
		}
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}


	// Measure versus Time
	if (parent->return_chinese(a,b+1) == "" && parent->is_category_non_recursive("Verb",a,b+2) == 1) {
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		if (whole_number > 1) { add_post_english(" times"); }
		else { add_post_english(" time"); }
		parent->make_only(a,b,this);
		parent->make_only(a,b+1,"Verb");
		myclass += ":Phrase";
	}
	if (parent->return_chinese(a,b-1) == "" && parent->is_category_non_recursive("Measure", a, b+1) == 1 && parent->is_category_non_recursive("Noun",a,b+2) == 1){
          	pre_pinyin += parent->text(a,b-1,0)->return_pinyin() + " ";
		shift_pre_chinese(parent,a,b-1,0,this); b--;
		add_post_english(" in excess of");
		parent->make_only(a,b,this);
		myclass += ":Phrase";
	}







	// Miscellaneous
	if (parent->return_chinese(a,b+1) == "ǿ") {
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		add_post_english(" strongest");
		if (parent->return_chinese(a,b-1) == "ǰ") {
          		pre_pinyin += parent->text(a,b+1,0)->return_pinyin() + " ";
			shift_pre_chinese(parent,a,b-1,0,this); b--;
			add_pre_english("top");
		}
		add_pre_english("the");
		parent->make_only(a,b,this);
		myclass += ":Phrase";
		plural_next = 0;
	}










	if (parent->return_chinese(a,b+1) == "") {
		if (parent->return_chinese(a,b-1).find("") != std::string::npos) {
          		post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
			shift_post_chinese(parent,a,b+1,0,this);
			english = suffix(english);
			english += " Congress";	
			parent->make_only(a,b,this);
			myclass += ":Phrase";
			plural_next = 0;
			myclass = "Unit:ProperNoun";
		}
	}
	if (parent->return_chinese(a,b+1) == "") {
          	post_pinyin += " " + parent->text(a,b+1,0)->return_pinyin();
		shift_post_chinese(parent,a,b+1,0,this);
		english += " points";	
		parent->make_only(a,b,this);
		myclass += ":Phrase";
		plural_next = 0;
		myclass += ":Stock";
	}




	if (parent->return_chinese(a,b-2) == "") {
		if (parent->is_category_non_recursive("Noun",a,b-1) == 1) {
			add_post_english(" from");
			add_post_english(parent->likely(a,b-1)->return_english());
			add_pre_english("a distance of");
          		pre_pinyin += parent->text(a,b-2,0)->return_pinyin();
          		pre_pinyin += parent->text(a,b-1,0)->return_pinyin();
			shift_pre_chinese(parent,a,b-1,0,this);b--;
			shift_pre_chinese(parent,a,b-1,0,this);b--;
			parent->make_only(a,b,this);
			myclass += ":Phrase";
		}
	}



	// Pluralize Following Nouns which do not require measure words
	std::string next = parent->return_chinese(a,b+1,0);
        if (
		next == "" ||
		next == "" ||
		next == "" ||
		next == "" ||
		next == "" ||
		next == "" ||
		next == "Сʱ" ||
		next == "" ||
		next == ""
	) {
		for (int i = 0; i < parent->elements->at(a)->at(b+1)->size(); i++) {
			parent->elements->at(a)->at(b+1)->at(i)->plural = 1;
		}
	}




	return 1;
}

int Number::advanced(Text *t, int a, int b, int c) {

  if (plural_next == 1) {
    myclass += ":Plural";
  }

  return 1;
}

void Number::generate_numbers() {

	int y = 0;
	int mm = 1;
	int max_loop = 0;
	int decimal_found  = 0;
	int decimal_start  = 0;
	int post_number = 0;

	double dmm = 0.1; 

	// Are We Consecutive?
	int starts_with_zero = 0;
	int previous_number = 0;
	int current_number = 0;
	int place_signifier_used = 0;	

	int temp;	

	long unsigned int whole_num = 0;
	double decimal_num = 0;
	long unsigned int multiplier = 0;
	long unsigned int decimal_multiplier = 0;
	float complete = 0;

	std::string fullchinese = return_numeric_chinese();
	following_pinyin = "";

	Number *post = this;


	// Generate Chinese for the Whole Number
	for (int i = 0, j = 1; j == 1 && post->post_chinese != NULL; i++) {
		if (post->post_chinese->chinese == "." || post->post_chinese->chinese_utf8s == "." || post->post_chinese->chinese == "" || post->post_chinese->chinese == "") {
			j = 0;
			max_loop--;
			decimal_found = 1;
			decimal_start = i+1;
			if (post->post_chinese->chinese == "") {
				post->post_chinese->english = ".";
			}
		} else {
			if (post->post_chinese->is_category_non_recursive("Number") != 1) {
//				max_loop--;
			}
		}
		max_loop++;
		post = (Number *) post->post_chinese;
	}


	// What is the first number?
	previous_number = convert2number(chinese);


	// Check to see if this starts with a ZERO
	if (previous_number == 0) { starts_with_zero = 1; }


	int prev_place_signifier = 0;
	int last_place_signifier = 0; 
	int first_number = 1;
	int ignore_first_zero = 0;

	// Now Generate Whole Number
	for (int i = 0; i <= max_loop; i++) { 

		post = this;
		for (int h = i; h < max_loop; h++) { post = (Number *) post->post_chinese; }

		if ( post->myclass.find("Number") == std::string::npos) {
		} else {


		temp = convert2number(post->chinese);


		// Check to see if this is a consecutive number
		current_number = temp;
		if (current_number == previous_number+1 && previous_number != 0) { 
			if (is_complex_number(fullchinese) != 1) {
			}
		}
		previous_number = current_number;




		// Check to See if the Number Has Grown Too Big -- Adjust
		if (mm == 1000000000 && whole_num == 0) {
		  mm = 1000;
		  add_post_english(" million");
  		}




	        if (temp == 0 && last_place_signifier == 1) { mm /= 10; }
		if (temp == 0) { 
                  mm *= 10; 
                  last_place_signifier = 0;
                }
		else {

			if (temp < 10) {
				if (last_place_signifier == 1) {

					mm/=10;
					whole_num = (int) whole_num + (int) (mm * temp);
					y = 0;
					last_place_signifier = 0;

				} else {

					// Only Add if Last is NOT a Place Signifier
  					if (last_place_signifier != 1 && first_number != 1) { mm *= 10; }
					whole_num = (int) whole_num + (int) (mm * temp);
					y = 0;
					last_place_signifier = 0;
				}
				first_number = 0;
			}
			else {
				if (mm < temp) { 
					if (prev_place_signifier == 0) {
						int temp2 = temp/10;
						int temp3 = whole_num/10;
						while (temp3 > 0) { temp2=temp2/10; temp3=temp3/10; }
						post_number = (whole_num * temp2) + post_number;
					}
					else {
						if (whole_num == 0) { whole_num = 1 * prev_place_signifier; }
						post_number = whole_num + post_number;
					}
					prev_place_signifier = temp;
					whole_num = 0;

					mm = temp *10; 
					multiplier = temp;
				}
				else { 

					if (mm == temp) {}
					else { mm *= temp; };

				}
				y = 1;
				place_signifier_used = 1;
				last_place_signifier = 1;

			}

		}
		}

	}



	if (y == 1) { whole_num = (int) (mm + whole_num) / 10; }
	if (post_number > 0 ) { whole_num += post_number; }




	int yi_in_decimal = 0;


	// Generate Chinese for Decimal
	mm = 1;
	if (decimal_found == 1) {

		post = this;
		// Generate Chinese for the Whole Number
		for (int i = 0; i < decimal_start; i++) { post = (Number *) post->post_chinese; }

		if (post->post_chinese == NULL) { return; }
		while (post->post_chinese != NULL) {
			post = (Number *) post->post_chinese;
			temp = convert2number(post->chinese);			
			if (temp > 9)  { 
				if (temp == 100000000) {
					// YI in DECIMAL NUMBER
					yi_in_decimal = 1;
				} else {
					if (mm == 1) { mm = temp; }
					else { mm *= temp; } 
				}
			}
			if (temp < 9) { decimal_num += (dmm * temp); dmm *= 0.1; }
		}
	}

	if (mm > 1) { decimal_multiplier = mm; }




	// Now Generate the English
	if (decimal_num == 0) {
		std::stringstream myStream1;
		myStream1 << whole_num;
		english = myStream1.str(); 

	} else {

		// Adjust Whole Number with Decimal Multiplier
		if (decimal_multiplier == 0) { decimal_multiplier = 1; }
			whole_num *= decimal_multiplier;
			int bigger_than_dec = (int) (decimal_num * (decimal_multiplier));
			if (bigger_than_dec <= 0) {}
			else {
				decimal_num = bigger_than_dec - (decimal_num * (decimal_multiplier));
                                decimal_num *= -1;
				if (decimal_num < 0.0001 && decimal_multiplier > 1) { decimal_num = 0; }
				whole_num += bigger_than_dec;
			}
		}


		// Handle Really Large Decimal Numbers with Yi
		if (yi_in_decimal == 1) {
			following_pinyin = "yi4";
			int testred = whole_num / 10;
			int testrem = whole_num - ((whole_num/10)*10);
			if (testrem != 0) {
				add_post_english(" hundred million");
			} else {

				testred = whole_num / 10000;
				testrem = whole_num - ((whole_num/10000)*10000);
				if (testrem != 0) {
					whole_num = (whole_num/10);
					decimal_num = (decimal_num/10);
					add_post_english(" billion");
				} else {
					whole_num = (whole_num/10000);
					add_post_english(" trillion");
				}
			}
		}



	std::stringstream myStream;
	if (decimal_num > 0) {
		myStream << whole_num; 
		myStream << decimal_num;
		english = myStream.str();
		english = english.substr(0, english.find("0.")) + "." + english.substr(english.find("0.")+2); 
		complete = (float) decimal_num + whole_num;
	} 
	else { 
		myStream << whole_num; 
		english = myStream.str(); 
	}


	whole_number = whole_num;
	decimal_number = decimal_num;
	full_number = complete;
	integer_representation = whole_number;
	decimal_representation = decimal_number;

	return;
}

int Number::convert2number(std::string word) {

	if (word == "") { return 0; }
	if (word == "") { return 0; }		
	if (word == "") { return 1; }
	if (word == "") { return 2; }
	if (word == "") { return 3; }
	if (word == "") { return 4; }
	if (word == "") { return 5; }
	if (word == "") { return 6; }
	if (word == "") { return 7; }
	if (word == "") { return 8; }
	if (word == "") { return 9; }
	if (word == "") { return 0; }
	if (word == "ʮ") { return 10; }
	if (word == "") { return 100; }
	if (word == "ǧ") { return 1000; }
	if (word == "")   { return 10000; }
	if (word == "ʮ") { return 100000; }
	if (word == "") { return 1000000; }
	if (word == "ǧ") { return 10000000; }
	if (word == "") { return 100000000; }
	if (word == "")   { return 100000000; }
	if (word == "һ") { return 1; }
	if (word == "") { return 2; }
	if (word == "") { return 2; }
	if (word == "") { return 3; }
	if (word == "") { return 4; }
	if (word == "") { return 5; }
	if (word == "") { return 6; }
	if (word == "") { return 7; }
	if (word == "") { return 8; }
	if (word == "") { return 9; }
	if (word == "1") { return 1; }
	if (word == "2") { return 2; }
	if (word == "3") { return 3; }
	if (word == "4") { return 4; }
	if (word == "5") { return 5; }
	if (word == "6") { return 6; }
	if (word == "7") { return 7; }
	if (word == "8") { return 8; }
	if (word == "9") { return 9; }
	if (word == "0") { return 0; }
	if (word == "Ҽ") { return 1; }
	if (word == "") { return 2; }
	if (word == "") { return 3; }
	if (word == "") { return 4; }
	if (word == "") { return 5; }
	if (word == "½") { return 6; }
	if (word == "") { return 7; }
	if (word == "") { return 8; }
	if (word == "") { return 9; }
	if (word == "ʰ") { return 10; }
	if (word == "") { return 100; }
	if (word == "Ǫ") { return 1000; }


	return -1;	
};

int Number::is_complex_number(std::string word) {

	if (word == "ʮ") { return 1; }
	if (word == "") { return 1; }
	if (word == "ǧ") { return 1; }
	if (word == "")   { return 1; }
	if (word == "")   { return 1; }

	return 0;	
};


/* Returns only the Numbers */
std::string Number::return_numeric_chinese() {
  std::string my_number = chinese;
  if (post_chinese != NULL) {
    if (post_chinese->is_category_non_recursive("Number") == 1) {
      Number *temp = (Number *) post_chinese;
      my_number += temp->return_numeric_chinese();
      return my_number;
    }
  }
  return my_number;
}





std::string Number::suffix(std::string x) {

	if (x.length() == 0) { return x; }
	if (x.substr(x.length()-1) == "1") { 
		if (x.length() >= 2) { if (x.substr(x.length()-2) == "11") { return x+"th"; } } return x+"st"; 
	}
	if (x.substr(x.length()-1) == "2") { 
		if (x.length() >= 2) { if (x.substr(x.length()-2) == "12") { return x+"th"; } } return x+"nd"; 
	}		
	if (x.substr(x.length()-1) == "3") { 
		if (x.length() >= 2) { if (x.substr(x.length()-2) == "13") { return x+"th"; } } return x+"rd"; 
	}		
	if (x.substr(x.length()-1) == "4") { return x+"th"; }		
	if (x.substr(x.length()-1) == "5") { return x+"th"; }		
	if (x.substr(x.length()-1) == "6") { return x+"th"; }		
	if (x.substr(x.length()-1) == "7") { return x+"th"; }		
	if (x.substr(x.length()-1) == "8") { return x+"th"; }		
	if (x.substr(x.length()-1) == "9") { return x+"th"; }		
	if (x.substr(x.length()-1) == "0") { return x+"th"; }		

	return x;	

};

std::string Number::add_commas_to_number(std::string numin) {

	std::string numout = numin;

	if (numin == "") { return ""; }
        if (numin.find("a") != std::string::npos) { return numin; }
        if (numin.find("e") != std::string::npos) { return numin; }
        if (numin.find("i") != std::string::npos) { return numin; }
        if (numin.find("o") != std::string::npos) { return numin; }
        if (numin.find("u") != std::string::npos) { return numin; }
        if (numin.find("st") != std::string::npos) { return numin; }
        if (numin.find("th") != std::string::npos) { return numin; }
        if (numin.find("nd") != std::string::npos) { return numin; }
        if (numin.find("rd") != std::string::npos) { return numin; }

	int decimal_location = numin.length()-1;
	if (numin.find(".") != std::string::npos) {
		decimal_location = numin.find(".")-1;
	}

	for (int i = decimal_location-3; i >= 0; i-=3) {
	numout = numout.substr(0, i+1) + "," + numout.substr(i+1);
	}

	return numout;
}
std::string Number::generate_complex_pinyin(int nu) {

	if (nu < 0) { return ""; }
	if (nu == 0) { return "ling2"; }

	std::string pp = "";
	std::string tt = "";
	std::string post_num = "";
	int i = 1;
	int last_num = 0;

	if (nu%100000000  == 0) { post_num += "yi4"; nu = nu/100000000; }
	if (nu%10000000 == 0) { post_num += "qian1wan4"; nu = nu / 10000000; }
	if (nu%1000000 == 0) { post_num += "bai3wan4"; nu = nu / 1000000; }
	if (nu%10000  == 0) { post_num += "wan4"; nu = nu / 10000; }

	while (nu > 0) {

		if (i == 100000) {

 			if (nu < 10000) {
				int nu2 = nu%10000;
				nu2 *= 10;
				nu = nu - nu%10000;
				pp = generate_complex_pinyin(nu2);
				if (pp.length() > 5) {
					if (
						 pp.substr(pp.length()-5) == "ling2"
					) {
						pp = pp.substr(0, pp.length()-5);
					}
				}
				if (pp == "ling2") { pp = ""; }
				i*= 10000;
			} else {
				int nu2 = (nu - nu%10000) / 10000;
				nu2 *= 10;
				pp = generate_complex_pinyin(nu2);
				if (pp == "ling2") { pp = ""; 
				} 
				else { pp += "yi4"; }
				i*= 10000;
				if (generate_complex_pinyin(nu%10000) != "ling2") {
					pp += generate_complex_pinyin(nu%10000);
				};
			}


		} else {

			pp = what_is_my_pinyin(nu%10);
			last_num = nu%10;
			nu -= nu%10;
			nu /= 10;

			if (pp == "yi1" && (i == 10000 || i == 100000000)) { pp = "yi2"; }

			if (pp == "ling2") { pp = ""; 

			}
			else {
				if (i == 10) { pp += "shi2"; }
				if (i == 100) { pp += "bai3 "; }
				if (i == 1000) { pp += "qian1 "; }
				if (i == 10000) { pp += "wan4 "; }
			}
				if (i == 100000000) { pp += "yi4 "; }

			i*=10;
		}
		tt = pp + tt;
	}

	if (post_num != "") { tt += post_num; }
	if (tt.find("yi1shi2") != std::string::npos) {	
		if (tt.length() > tt.find("yi1shi2"+6)) {
			tt = tt.substr(0, tt.find("yi1shi2")) + "shi2" + tt.substr(tt.find("yi1shi2")+7);
		}
		else { 
			tt = tt.substr(0, tt.find("yi1shi2")) + "shi2";
		}
	}


	return tt;


}
std::string Number::what_is_my_pinyin(int x) {
	if (x == 1) { return "yi1"; }
	if (x == 2) { if (chinese == "") {return "liang3";} return "er4"; }
	if (x == 3) { return "san1"; }
	if (x == 4) { return "si4"; }
	if (x == 5) { return "wu3"; }
	if (x == 6) { return "liu4"; }
	if (x == 7) { return "qi1"; }
	if (x == 8) { return "ba1"; }
	if (x == 9) { return "jiu3"; }
	if (x == 0) { return "ling2"; }
	return "";
}





